public inbox for passt-dev@passt.top
 help / color / mirror / code / Atom feed
From: David Gibson <david@gibson.dropbear.id.au>
To: passt-dev@passt.top, Stefano Brivio <sbrivio@redhat.com>
Cc: David Gibson <david@gibson.dropbear.id.au>
Subject: [PATCH 06/32] tcp_splice: Helpers for converting from index to/from tcp_splice_conn
Date: Wed, 16 Nov 2022 15:41:46 +1100	[thread overview]
Message-ID: <20221116044212.3876516-7-david@gibson.dropbear.id.au> (raw)
In-Reply-To: <20221116044212.3876516-1-david@gibson.dropbear.id.au>

Like we already have for non-spliced connections, create a CONN_IDX()
macro for looking up the index of spliced connection structures.  Change
the name of the array of spliced connections to be different from that for
non-spliced connections (even though they're in different modules).  This
will make subsequent changes a bit safer.

Signed-off-by: David Gibson <david@gibson.dropbear.id.au>
---
 tcp_splice.c | 43 +++++++++++++++++++++++++------------------
 1 file changed, 25 insertions(+), 18 deletions(-)

diff --git a/tcp_splice.c b/tcp_splice.c
index 3c5c111..4cc4ad2 100644
--- a/tcp_splice.c
+++ b/tcp_splice.c
@@ -13,10 +13,11 @@
  * DOC: Theory of Operation
  *
  *
  * For local traffic directed to TCP ports configured for direct mapping between
  * namespaces, packets are directly translated between L4 sockets using a pair
  * of splice() syscalls. These connections are tracked in the @tc array of
  * struct tcp_splice_conn, using these events:
  *
  * - SPLICE_CONNECT:		connection accepted, connecting to target
  * - SPLICE_ESTABLISHED:	connection to target established
@@ -113,10 +114,11 @@ struct tcp_splice_conn {
 #define CONN_V6(x)			(x->flags & SOCK_V6)
 #define CONN_V4(x)			(!CONN_V6(x))
 #define CONN_HAS(conn, set)		((conn->events & (set)) == (set))
-#define CONN(index)			(tc + (index))
+#define CONN(index)			(tc_splice + (index))
+#define CONN_IDX(conn)			((conn) - tc_splice)
 
 /* Spliced connections */
-static struct tcp_splice_conn tc[TCP_SPLICE_MAX_CONNS];
+static struct tcp_splice_conn tc_splice[TCP_SPLICE_MAX_CONNS];
 
 /* Display strings for connection events */
 static const char *tcp_splice_event_str[] __attribute((__unused__)) = {
@@ -173,7 +175,7 @@ static void conn_flag_do(const struct ctx *c, struct tcp_splice_conn *conn,
 
 		conn->flags &= flag;
 		if (fls(~flag) >= 0) {
-			debug("TCP (spliced): index %li: %s dropped", conn - tc,
+			debug("TCP (spliced): index %li: %s dropped", CONN_IDX(conn),
 			      tcp_splice_flag_str[fls(~flag)]);
 		}
 	} else {
@@ -182,7 +184,7 @@ static void conn_flag_do(const struct ctx *c, struct tcp_splice_conn *conn,
 
 		conn->flags |= flag;
 		if (fls(flag) >= 0) {
-			debug("TCP (spliced): index %li: %s", conn - tc,
+			debug("TCP (spliced): index %li: %s", CONN_IDX(conn),
 			      tcp_splice_flag_str[fls(flag)]);
 		}
 	}
@@ -211,11 +213,11 @@ static int tcp_splice_epoll_ctl(const struct ctx *c,
 	int m = (conn->flags & IN_EPOLL) ? EPOLL_CTL_MOD : EPOLL_CTL_ADD;
 	union epoll_ref ref_a = { .r.proto = IPPROTO_TCP, .r.s = conn->a,
 				  .r.p.tcp.tcp.splice = 1,
-				  .r.p.tcp.tcp.index = conn - tc,
+				  .r.p.tcp.tcp.index = CONN_IDX(conn),
 				  .r.p.tcp.tcp.v6 = CONN_V6(conn) };
 	union epoll_ref ref_b = { .r.proto = IPPROTO_TCP, .r.s = conn->b,
 				  .r.p.tcp.tcp.splice = 1,
-				  .r.p.tcp.tcp.index = conn - tc,
+				  .r.p.tcp.tcp.index = CONN_IDX(conn),
 				  .r.p.tcp.tcp.v6 = CONN_V6(conn) };
 	struct epoll_event ev_a = { .data.u64 = ref_a.u64 };
 	struct epoll_event ev_b = { .data.u64 = ref_b.u64 };
@@ -257,7 +259,7 @@ static void conn_event_do(const struct ctx *c, struct tcp_splice_conn *conn,
 
 		conn->events &= event;
 		if (fls(~event) >= 0) {
-			debug("TCP (spliced): index %li, ~%s", conn - tc,
+			debug("TCP (spliced): index %li, ~%s", CONN_IDX(conn),
 			      tcp_splice_event_str[fls(~event)]);
 		}
 	} else {
@@ -266,7 +268,7 @@ static void conn_event_do(const struct ctx *c, struct tcp_splice_conn *conn,
 
 		conn->events |= event;
 		if (fls(event) >= 0) {
-			debug("TCP (spliced): index %li, %s", conn - tc,
+			debug("TCP (spliced): index %li, %s", CONN_IDX(conn),
 			      tcp_splice_event_str[fls(event)]);
 		}
 	}
@@ -292,8 +294,8 @@ static void tcp_table_splice_compact(struct ctx *c,
 {
 	struct tcp_splice_conn *move;
 
-	if ((hole - tc) == --c->tcp.splice_conn_count) {
-		debug("TCP (spliced): index %li (max) removed", hole - tc);
+	if (CONN_IDX(hole) == --c->tcp.splice_conn_count) {
+		debug("TCP (spliced): index %li (max) removed", CONN_IDX(hole));
 		return;
 	}
 
@@ -307,7 +309,8 @@ static void tcp_table_splice_compact(struct ctx *c,
 	move->pipe_b_a[0] = move->pipe_b_a[1] = -1;
 	move->flags = move->events = 0;
 
-	debug("TCP (spliced): index %li moved to %li", move - tc, hole - tc);
+	debug("TCP (spliced): index %li moved to %li",
+	      CONN_IDX(move), CONN_IDX(hole));
 	tcp_splice_epoll_ctl(c, hole);
 	if (tcp_splice_epoll_ctl(c, hole))
 		conn_flag(c, hole, CLOSING);
@@ -345,7 +348,7 @@ static void tcp_splice_destroy(struct ctx *c, struct tcp_splice_conn *conn)
 
 	conn->events = CLOSED;
 	conn->flags = 0;
-	debug("TCP (spliced): index %li, CLOSED", conn - tc);
+	debug("TCP (spliced): index %li, CLOSED", CONN_IDX(conn));
 
 	tcp_table_splice_compact(c, conn);
 }
@@ -872,7 +875,9 @@ void tcp_splice_timer(struct ctx *c)
 {
 	struct tcp_splice_conn *conn;
 
-	for (conn = CONN(c->tcp.splice_conn_count - 1); conn >= tc; conn--) {
+	for (conn = CONN(c->tcp.splice_conn_count - 1);
+	     conn >= tc_splice;
+	     conn--) {
 		if (conn->flags & CLOSING) {
 			tcp_splice_destroy(c, conn);
 			return;
@@ -918,7 +923,9 @@ void tcp_splice_defer_handler(struct ctx *c)
 	if (c->tcp.splice_conn_count < MIN(max_files / 6, max_conns))
 		return;
 
-	for (conn = CONN(c->tcp.splice_conn_count - 1); conn >= tc; conn--) {
+	for (conn = CONN(c->tcp.splice_conn_count - 1);
+	     conn >= tc_splice;
+	     conn--) {
 		if (conn->flags & CLOSING)
 			tcp_splice_destroy(c, conn);
 	}
@@ -13,10 +13,11 @@
  *
  *
- * For local traffic directed to TCP ports configured for direct mapping between
- * namespaces, packets are directly translated between L4 sockets using a pair
- * of splice() syscalls. These connections are tracked in the @tc array of
- * struct tcp_splice_conn, using these events:
+ * For local traffic directed to TCP ports configured for direct
+ * mapping between namespaces, packets are directly translated between
+ * L4 sockets using a pair of splice() syscalls. These connections are
+ * tracked in the @tc_splice array of struct tcp_splice_conn, using
+ * these events:
  *
  * - SPLICE_CONNECT:		connection accepted, connecting to target
-- 
@@ -13,10 +13,11 @@
  * DOC: Theory of Operation
  *
  *
- * For local traffic directed to TCP ports configured for direct mapping between
- * namespaces, packets are directly translated between L4 sockets using a pair
- * of splice() syscalls. These connections are tracked in the @tc array of
- * struct tcp_splice_conn, using these events:
+ * For local traffic directed to TCP ports configured for direct
+ * mapping between namespaces, packets are directly translated between
+ * L4 sockets using a pair of splice() syscalls. These connections are
+ * tracked in the @tc_splice array of struct tcp_splice_conn, using
+ * these events:
  *
  * - SPLICE_CONNECT:		connection accepted, connecting to target
  * - SPLICE_ESTABLISHED:	connection to target established
@@ -113,10 +114,11 @@ struct tcp_splice_conn {
 #define CONN_V6(x)			(x->flags & SOCK_V6)
 #define CONN_V4(x)			(!CONN_V6(x))
 #define CONN_HAS(conn, set)		((conn->events & (set)) == (set))
-#define CONN(index)			(tc + (index))
+#define CONN(index)			(tc_splice + (index))
+#define CONN_IDX(conn)			((conn) - tc_splice)
 
 /* Spliced connections */
-static struct tcp_splice_conn tc[TCP_SPLICE_MAX_CONNS];
+static struct tcp_splice_conn tc_splice[TCP_SPLICE_MAX_CONNS];
 
 /* Display strings for connection events */
 static const char *tcp_splice_event_str[] __attribute((__unused__)) = {
@@ -173,7 +175,7 @@ static void conn_flag_do(const struct ctx *c, struct tcp_splice_conn *conn,
 
 		conn->flags &= flag;
 		if (fls(~flag) >= 0) {
-			debug("TCP (spliced): index %li: %s dropped", conn - tc,
+			debug("TCP (spliced): index %li: %s dropped", CONN_IDX(conn),
 			      tcp_splice_flag_str[fls(~flag)]);
 		}
 	} else {
@@ -182,7 +184,7 @@ static void conn_flag_do(const struct ctx *c, struct tcp_splice_conn *conn,
 
 		conn->flags |= flag;
 		if (fls(flag) >= 0) {
-			debug("TCP (spliced): index %li: %s", conn - tc,
+			debug("TCP (spliced): index %li: %s", CONN_IDX(conn),
 			      tcp_splice_flag_str[fls(flag)]);
 		}
 	}
@@ -211,11 +213,11 @@ static int tcp_splice_epoll_ctl(const struct ctx *c,
 	int m = (conn->flags & IN_EPOLL) ? EPOLL_CTL_MOD : EPOLL_CTL_ADD;
 	union epoll_ref ref_a = { .r.proto = IPPROTO_TCP, .r.s = conn->a,
 				  .r.p.tcp.tcp.splice = 1,
-				  .r.p.tcp.tcp.index = conn - tc,
+				  .r.p.tcp.tcp.index = CONN_IDX(conn),
 				  .r.p.tcp.tcp.v6 = CONN_V6(conn) };
 	union epoll_ref ref_b = { .r.proto = IPPROTO_TCP, .r.s = conn->b,
 				  .r.p.tcp.tcp.splice = 1,
-				  .r.p.tcp.tcp.index = conn - tc,
+				  .r.p.tcp.tcp.index = CONN_IDX(conn),
 				  .r.p.tcp.tcp.v6 = CONN_V6(conn) };
 	struct epoll_event ev_a = { .data.u64 = ref_a.u64 };
 	struct epoll_event ev_b = { .data.u64 = ref_b.u64 };
@@ -257,7 +259,7 @@ static void conn_event_do(const struct ctx *c, struct tcp_splice_conn *conn,
 
 		conn->events &= event;
 		if (fls(~event) >= 0) {
-			debug("TCP (spliced): index %li, ~%s", conn - tc,
+			debug("TCP (spliced): index %li, ~%s", CONN_IDX(conn),
 			      tcp_splice_event_str[fls(~event)]);
 		}
 	} else {
@@ -266,7 +268,7 @@ static void conn_event_do(const struct ctx *c, struct tcp_splice_conn *conn,
 
 		conn->events |= event;
 		if (fls(event) >= 0) {
-			debug("TCP (spliced): index %li, %s", conn - tc,
+			debug("TCP (spliced): index %li, %s", CONN_IDX(conn),
 			      tcp_splice_event_str[fls(event)]);
 		}
 	}
@@ -292,8 +294,8 @@ static void tcp_table_splice_compact(struct ctx *c,
 {
 	struct tcp_splice_conn *move;
 
-	if ((hole - tc) == --c->tcp.splice_conn_count) {
-		debug("TCP (spliced): index %li (max) removed", hole - tc);
+	if (CONN_IDX(hole) == --c->tcp.splice_conn_count) {
+		debug("TCP (spliced): index %li (max) removed", CONN_IDX(hole));
 		return;
 	}
 
@@ -307,7 +309,8 @@ static void tcp_table_splice_compact(struct ctx *c,
 	move->pipe_b_a[0] = move->pipe_b_a[1] = -1;
 	move->flags = move->events = 0;
 
-	debug("TCP (spliced): index %li moved to %li", move - tc, hole - tc);
+	debug("TCP (spliced): index %li moved to %li",
+	      CONN_IDX(move), CONN_IDX(hole));
 	tcp_splice_epoll_ctl(c, hole);
 	if (tcp_splice_epoll_ctl(c, hole))
 		conn_flag(c, hole, CLOSING);
@@ -345,7 +348,7 @@ static void tcp_splice_destroy(struct ctx *c, struct tcp_splice_conn *conn)
 
 	conn->events = CLOSED;
 	conn->flags = 0;
-	debug("TCP (spliced): index %li, CLOSED", conn - tc);
+	debug("TCP (spliced): index %li, CLOSED", CONN_IDX(conn));
 
 	tcp_table_splice_compact(c, conn);
 }
@@ -872,7 +875,9 @@ void tcp_splice_timer(struct ctx *c)
 {
 	struct tcp_splice_conn *conn;
 
-	for (conn = CONN(c->tcp.splice_conn_count - 1); conn >= tc; conn--) {
+	for (conn = CONN(c->tcp.splice_conn_count - 1);
+	     conn >= tc_splice;
+	     conn--) {
 		if (conn->flags & CLOSING) {
 			tcp_splice_destroy(c, conn);
 			return;
@@ -918,7 +923,9 @@ void tcp_splice_defer_handler(struct ctx *c)
 	if (c->tcp.splice_conn_count < MIN(max_files / 6, max_conns))
 		return;
 
-	for (conn = CONN(c->tcp.splice_conn_count - 1); conn >= tc; conn--) {
+	for (conn = CONN(c->tcp.splice_conn_count - 1);
+	     conn >= tc_splice;
+	     conn--) {
 		if (conn->flags & CLOSING)
 			tcp_splice_destroy(c, conn);
 	}
-- 
2.38.1


  parent reply	other threads:[~2022-11-16  4:42 UTC|newest]

Thread overview: 57+ messages / expand[flat|nested]  mbox.gz  Atom feed  top
2022-11-16  4:41 [PATCH 00/32] Use dual stack sockets to listen for inbound TCP connections David Gibson
2022-11-16  4:41 ` [PATCH 01/32] clang-tidy: Suppress warning about assignments in if statements David Gibson
2022-11-16 23:10   ` Stefano Brivio
2022-11-17  1:20     ` David Gibson
2022-11-16  4:41 ` [PATCH 02/32] style: Minor corrections to function comments David Gibson
2022-11-16 23:11   ` Stefano Brivio
2022-11-17  1:21     ` David Gibson
2022-11-16  4:41 ` [PATCH 03/32] tcp_splice: #include tcp_splice.h in tcp_splice.c David Gibson
2022-11-16  4:41 ` [PATCH 04/32] tcp: Remove unused TCP_MAX_SOCKS constant David Gibson
2022-11-16  4:41 ` [PATCH 05/32] tcp: Better helpers for converting between connection pointer and index David Gibson
2022-11-16 23:11   ` Stefano Brivio
2022-11-17  1:24     ` David Gibson
2022-11-16  4:41 ` David Gibson [this message]
2022-11-16  4:41 ` [PATCH 07/32] tcp: Move connection state structures into a shared header David Gibson
2022-11-16  4:41 ` [PATCH 08/32] tcp: Add connection union type David Gibson
2022-11-16  4:41 ` [PATCH 09/32] tcp: Improved helpers to update connections after moving David Gibson
2022-11-16  4:41 ` [PATCH 10/32] tcp: Unify spliced and non-spliced connection tables David Gibson
2022-11-16  4:41 ` [PATCH 11/32] tcp: Unify tcp_defer_handler and tcp_splice_defer_handler() David Gibson
2022-11-16  4:41 ` [PATCH 12/32] tcp: Partially unify tcp_timer() and tcp_splice_timer() David Gibson
2022-11-16  4:41 ` [PATCH 13/32] tcp: Unify the IN_EPOLL flag David Gibson
2022-11-16  4:41 ` [PATCH 14/32] tcp: Separate helpers to create ns listening sockets David Gibson
2022-11-16 23:51   ` Stefano Brivio
2022-11-17  1:32     ` David Gibson
2022-11-16  4:41 ` [PATCH 15/32] tcp: Unify part of spliced and non-spliced conn_from_sock path David Gibson
2022-11-16 23:53   ` Stefano Brivio
2022-11-17  1:37     ` David Gibson
2022-11-17  7:30       ` Stefano Brivio
2022-11-17  8:58         ` David Gibson
2022-11-16  4:41 ` [PATCH 16/32] tcp: Use the same sockets to listen for spliced and non-spliced connections David Gibson
2022-11-16 23:54   ` Stefano Brivio
2022-11-17  1:43     ` David Gibson
2022-11-16  4:41 ` [PATCH 17/32] tcp: Remove splice from tcp_epoll_ref David Gibson
2022-11-16  4:41 ` [PATCH 18/32] tcp: Don't store hash bucket in connection structures David Gibson
2022-11-16  4:41 ` [PATCH 19/32] inany: Helper functions for handling addresses which could be IPv4 or IPv6 David Gibson
2022-11-16 23:54   ` Stefano Brivio
2022-11-17  1:48     ` David Gibson
2022-11-16  4:42 ` [PATCH 20/32] tcp: Hash IPv4 and IPv4-mapped-IPv6 addresses the same David Gibson
2022-11-16  4:42 ` [PATCH 21/32] tcp: Take tcp_hash_insert() address from struct tcp_conn David Gibson
2022-11-16  4:42 ` [PATCH 22/32] tcp: Simplify tcp_hash_match() to take an inany_addr David Gibson
2022-11-16  4:42 ` [PATCH 23/32] tcp: Unify initial sequence number calculation for IPv4 and IPv6 David Gibson
2022-11-16  4:42 ` [PATCH 24/32] tcp: Have tcp_seq_init() take its parameters from struct tcp_conn David Gibson
2022-11-16  4:42 ` [PATCH 25/32] tcp: Fix small errors in tcp_seq_init() time handling David Gibson
2022-11-16  4:42 ` [PATCH 26/32] tcp: Remove v6 flag from tcp_epoll_ref David Gibson
2022-11-17  0:15   ` Stefano Brivio
2022-11-17  1:50     ` David Gibson
2022-11-16  4:42 ` [PATCH 27/32] tcp: NAT IPv4-mapped IPv6 addresses like IPv4 addresses David Gibson
2022-11-17  0:15   ` Stefano Brivio
2022-11-17  2:00     ` David Gibson
2022-11-16  4:42 ` [PATCH 28/32] tcp_splice: Allow splicing of connections from IPv4-mapped loopback David Gibson
2022-11-17  0:15   ` Stefano Brivio
2022-11-17  2:05     ` David Gibson
2022-11-16  4:42 ` [PATCH 29/32] tcp: Consolidate tcp_sock_init[46] David Gibson
2022-11-16  4:42 ` [PATCH 30/32] util: Allow sock_l4() to open dual stack sockets David Gibson
2022-11-16  4:42 ` [PATCH 31/32] util: Always return -1 on error in sock_l4() David Gibson
2022-11-16  4:42 ` [PATCH 32/32] tcp: Use dual stack sockets for port forwarding when possible David Gibson
2022-11-17  0:15   ` Stefano Brivio
2022-11-17  2:08     ` David Gibson

Reply instructions:

You may reply publicly to this message via plain-text email
using any one of the following methods:

* Save the following mbox file, import it into your mail client,
  and reply-to-all from there: mbox

  Avoid top-posting and favor interleaved quoting:
  https://en.wikipedia.org/wiki/Posting_style#Interleaved_style

* Reply using the --to, --cc, and --in-reply-to
  switches of git-send-email(1):

  git send-email \
    --in-reply-to=20221116044212.3876516-7-david@gibson.dropbear.id.au \
    --to=david@gibson.dropbear.id.au \
    --cc=passt-dev@passt.top \
    --cc=sbrivio@redhat.com \
    /path/to/YOUR_REPLY

  https://kernel.org/pub/software/scm/git/docs/git-send-email.html

* If your mail client supports setting the In-Reply-To header
  via mailto: links, try the mailto: link
Be sure your reply has a Subject: header at the top and a blank line before the message body.
Code repositories for project(s) associated with this public inbox

	https://passt.top/passt

This is a public inbox, see mirroring instructions
for how to clone and mirror all data and code used for this inbox;
as well as URLs for IMAP folder(s).